/*
 * Copyright 2015 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

package fr.ill.ics.cameo.test;

import fr.ill.ics.cameo.Application;
import fr.ill.ics.cameo.Application.This;
import fr.ill.ics.cameo.RemoteException;


public class TestCancel {

	public static void main(String[] args) {

		Application.This.init(args);
		
		try {
			// Test This.cancelWaitings.
			{
				System.out.println("Starting stopjava for cancelWaitings");
				
				Application.Instance stopApplication = This.getServer().start("stopjava");
	
				// Start thread.
				Thread cancel = new Thread(new Runnable() {
					@Override
				    public void run() {
				    	try {
				    		Thread.sleep(1000);
				    		This.cancelWaitings();
				    		
				    	} catch (InterruptedException e) {
						}
				    }
				});
				
				cancel.start();
				
				stopApplication.waitFor();
	
				System.out.println("End of waitFor");
	
				stopApplication.stop();
				int state = stopApplication.waitFor();
	
				System.out.println("End of stopjava with state " + Application.State.toString(state));
	
				cancel.join();
			}
			
			// Test Instance.cancelWaitFor.
			{
				System.out.println("Starting stopjava for cancelWaitFor");
				
				final Application.Instance stopApplication = This.getServer().start("stopjava");
	
				// Start thread
				Thread cancel = new Thread(new Runnable() {
					@Override
				    public void run() {
				    	try {
				    		Thread.sleep(1000);
				    		stopApplication.cancelWaitFor();
				    		
				    	} catch (InterruptedException e) {
						}
				    }
				});
				
				cancel.start();
				
				stopApplication.waitFor();
	
				System.out.println("End of waitFor");
	
				stopApplication.stop();
				int state = stopApplication.waitFor();
	
				System.out.println("End of stopjava with state " + Application.State.toString(state));
	
				cancel.join();
			}
			
			// Test Publisher.cancelWaitForSubscribers.
			{
				System.out.println("Creating publisher and waiting for 1 subscriber...");
				
				// create the publisher
				final Application.Publisher publisher = Application.Publisher.create("publisher", 1);
				
				Thread cancel = new Thread(new Runnable() {
					@Override
				    public void run() {
				    	try {
				    		Thread.sleep(1000);
				    		publisher.cancelWaitForSubscribers();	
				    		
				    	} catch (InterruptedException e) {
						}
				    }
				});
				
				cancel.start();
				
				System.out.println("Wait for subscribers");
				
				boolean synced = publisher.waitForSubscribers();
							
				cancel.join();
				
				System.out.println("Synchronization with the subscriber " + synced);
			}
			
			// Test the killing of the application.
			{
				System.out.println("Starting publisherloopjava for killing");
				
				final Application.Instance pubLoopApplication = This.getServer().start("publisherloopjava");
	
				// Start thread
				Thread cancel = new Thread(new Runnable() {
					@Override
				    public void run() {
				    	try {
				    		Thread.sleep(1000);
				    		pubLoopApplication.kill();
				    		
				    	} catch (InterruptedException e) {
						}
				    }
				});
				
				cancel.start();
				
				Application.Subscriber subscriber = Application.Subscriber.create(pubLoopApplication, "publisher");
								
				while (true) {
					String data = subscriber.receiveString();
					if (data == null) {
						System.out.println("Exiting loop");
						break;
						
					} else {
						System.out.println("Received " + data);
					}
				}
				
				System.out.println("Subscriber end of stream " + subscriber.isEnded());

				int state = pubLoopApplication.waitFor();

				System.out.println("End of waitFor " + Application.State.toString(state));
	
				cancel.join();
			}
			
			try {
	    		Thread.sleep(1000);
	    		
	    	} catch (InterruptedException e) {
			}
			
			// Test the Responder.
			{
				System.out.println("Creating responder and waiting for requests");
				
				// Create the publisher.
				final Application.Responder responder = Application.Responder.create("responder");
				
				Thread cancel = new Thread(new Runnable() {
					@Override
				    public void run() {
				    	try {
				    		Thread.sleep(1000);
				    		responder.cancel();	
				    		
				    	} catch (InterruptedException e) {
						}
				    }
				});
				
				cancel.start();
				
				System.out.println("Wait for requests");
				
				Application.Request request = responder.receive();
				
				if (request != null) {
					System.err.println("Responder error: receive should return null");		
				}
				
				cancel.join();
			}
			
		} catch (RemoteException e) {
			System.err.println("Publisher error");
			
		} catch (InterruptedException e) {

		} finally {
			Application.This.terminate();			
		}
		
		System.out.println("Finished the application");
	}

}